@@ -1,7 +1,9 @@
 from .descale import *  # noqa: F401, F403
 from .exceptions import *  # noqa: F401, F403
+from .funcs import *  # noqa: F401, F403
 from .gamma import *  # noqa: F401, F403
+from .helpers import *  # noqa: F401, F403
 from .mask import *  # noqa: F401, F403
 from .scale import *  # noqa: F401, F403
+from .shaders import *  # noqa: F401, F403
 from .types import *  # noqa: F401, F403
-from .utils import *  # noqa: F401, F403
