@@ -25,6 +25,7 @@ __all__ = [
 
 class MergeScalers(GenericScaler):
     def __init__(self, *scalers: ScalerT | tuple[ScalerT, float | None]) -> None:
+        """Create a unified Scaler from multiple Scalers with optional weights."""
         if (l := len(scalers)) < 2:
             raise CustomIndexError(f'Not enough scalers passed! ({l})', self.__class__)
         elif len(scalers) > len(EXPR_VARS):
@@ -85,21 +86,34 @@ class MergeScalers(GenericScaler):
 
 @dataclass
 class ClampScaler(GenericScaler):
+    """Clamp a reference Scaler."""
+
     ref_scaler: ScalerT
+    """Scaler to clamp."""
 
     strength: int = 80
+    """Strength of clamping."""
 
     overshoot: float | None = None
+    """Overshoot threshold."""
+
     undershoot: float | None = None
+    """Undershoot threshold."""
 
     limit: RepairMode | bool = True
+    """Whether to use under/overshoot limit (True) or a reference repaired clip for limiting."""
 
     operator: Literal[ExprOp.MAX, ExprOp.MIN] | None = ExprOp.MIN
+    """Whether to take the brightest or darkest pixels in the merge."""
+
     masked: bool = True
+    """Whether to mask with a ringing mask or not."""
 
     reference: ScalerT | vs.VideoNode = Nnedi3(0, opencl=None)
+    """Reference Scaler used to clamp ref_scaler"""
 
     range_out: ColorRange | None = None
+    """Range out for clamping the output. If None it will be fetched from the VideoNode."""
 
     def __post_init__(self) -> None:
         super().__post_init__()
@@ -177,6 +191,8 @@ class ClampScaler(GenericScaler):
 
 
 class UnsharpLimitScaler(GenericScaler):
+    """Limit a scaler with a masked unsharping."""
+
     def __init__(
         self, ref_scaler: ScalerT,
         unsharp_func: Callable[
@@ -186,6 +202,14 @@ class UnsharpLimitScaler(GenericScaler):
         reference: ScalerT | vs.VideoNode = Nnedi3(0, opencl=None),
         *args: P.args, **kwargs: P.kwargs
     ) -> None:
+        """
+        :param ref_scaler:      Scaler of which to limit haloing.
+        :param unsharp_func:    Unsharpening function used as reference for limiting.
+        :param merge_mode:      Whether to limit with LimitFilterMode,
+                                use a median filter (True) or just take the darkest pixels (False).
+        :param reference:       Reference scaler used to fill in the haloed parts.
+        """
+
         self.unsharp_func = unsharp_func
 
         self.merge_mode = merge_mode
@@ -228,10 +252,14 @@ class UnsharpLimitScaler(GenericScaler):
 
 @dataclass
 class MergedFSRCNNX(ClampScaler):
+    """Clamped FSRCNNX Scaler."""
+
     ref_scaler: FSRCNNXShaderT = field(default_factory=lambda: FSRCNNXShader.x56, kw_only=True)
 
 
 class UnsharpedFSRCNNX(UnsharpLimitScaler):
+    """Clamped FSRCNNX Scaler with an unsharp mask."""
+
     def __init__(
         self,
         unsharp_func: Callable[
