@@ -17,6 +17,24 @@ def descale_detail_mask(
     clip: vs.VideoNode, rescaled: vs.VideoNode, thr: float = 0.05,
     inflate: int = 2, xxpand: tuple[int, int] = (4, 0)
 ) -> vs.VideoNode:
+    """
+    Mask non-native resolution detail to prevent detail loss and artifacting.
+
+    Descaling without masking is very dangerous, as descaling FHD material often leads to
+    heavy artifacting and fine detail loss.
+
+    :param clip:        Original clip.
+    :param rescaled:    Clip rescaled using the presumed native kernel.
+    :param thr:         Binarizing threshold. Lower will catch more.
+                        Assumes float bitdepth input.
+                        Default: 0.05.
+    :param inflate:     Amount of times to ``inflate`` the mask. Default: 2.
+    :param xxpand:      Amount of times to ``Maximum`` the clip by.
+                        The first ``Maximum`` is done before inflating, the second after.
+                        Default: 4 times pre-inflating, 0 times post-inflating.
+
+    :return:            Mask containing all the native FHD detail.
+    """
     mask = norm_expr([get_y(clip), get_y(rescaled)], 'x y - abs')
 
     mask = mask.std.Binarize(thr * get_peak_value(mask))
@@ -39,6 +57,19 @@ def descale_error_mask(
     expands: int | tuple[int, int, int] = (2, 2, 3),
     blur: int | float = 3, bwbias: int = 1, tr: int = 1
 ) -> vs.VideoNode:
+    """
+    Create an error mask from the original and rescaled clip.
+
+    :param clip:        Original clip.
+    :param rescaled:    Rescaled clip.
+    :param thr:         Threshold of the minimum difference.
+    :param expands:     Iterations of mask expand at each step (diff, expand, binarize).
+    :param blur:        How much to blur the clip. If int, it will be a box_blur, else gauss_blur.
+    :param bwbias:      Calculate a bias with the clip's chroma.
+    :param tr:          Make the error mask temporally stable with a temporal radius.
+
+    :return:            Descale error mask.
+    """
     assert clip.format and rescaled.format
 
     y, *chroma = split(clip)
